
/* mostly borrwed from Nick Trown (COW) */

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#include <ctype.h>
#ifdef hpux
#include <time.h>
#else				/* hpux */
#include <sys/time.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>

#include "netrek.h"

#define KEYS		4
#define PARADISE	'P'
#define MAX_SERVERS	32

struct servers {
   char            address[BUFSIZ];
   int             port;
   int             time;
   int             players;
   int             status;
   char            flag;
   char            def;
   int             RSA_client;
};

static char    *metaserver = "metaserver.ecst.csuchico.edu";
static int      metaport = 3521;
static struct servers *serverlist;
static int      num_servers;
static W_Window metaWin;
static int      quit_meta;

static char    *keystrings[KEYS] =
{"OPEN:", "Wait queue:", "Nobody", "Not responding"};

#define KEY_OPEN		0
#define KEY_WAITQ		1
#define KEY_NOBODY		2
#define KEY_NORESP		3
#define KEY_UNKNOWN		4	/* special case */

#if __STDC__ || defined(__cplusplus)
#define P_(s) s
#else
#define P_(s) ()
#endif

/* metaserver.c */
static int getnumber P_((char *string, int start));
static int open_port P_((char *host, int port, int verbose));
static void readmeta P_((int sock));
static void parsemeta P_((void));
static void metawindow P_((void));
static void metarefresh P_((int i));
static void metaaction P_((W_Event * data));
static void metadone P_((void));

#undef P_

static int
getnumber(string, start)
    char           *string;
    int             start;
{
   char            temp[BUFSIZ];
   int             c;
   int             tc = 0;

   for (c = start; c <= strlen(string); c++) {
      if ((string[c] >= '0') && (string[c] <= '9')) {
	 temp[tc++] = string[c];
      } else if (tc > 0) {
	 temp[tc] = '\0';
	 return (atoi(temp));
      }
   }
   return 0;
}

/*
 * The connection to the metaserver is by Andy McFadden. This calls the
 * metaserver and parses the output into something useful
 */

static int
open_port(host, port, verbose)
    char           *host;
    int             port;
    int             verbose;
{
   struct sockaddr_in addr;
   struct hostent *hp;
   int             sock;

   /* Connect to the metaserver */
   /* get numeric form */
   if ((addr.sin_addr.s_addr = inet_addr(host)) == -1) {
      if ((hp = gethostbyname(host)) == NULL) {
	 if (verbose)
	    fprintf(stderr, "netrek: unknown host '%s'\n", host);
	 return (-1);
      } else {
	 addr.sin_addr.s_addr = *(long *) hp->h_addr;
      }
   }
   addr.sin_family = AF_INET;
   addr.sin_port = htons(port);
   if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
      if (verbose)
	 perror("socket");
      return (-1);
   }
   if (connect(sock, (struct sockaddr *) & addr, sizeof(addr)) < 0) {
      if (verbose){
	 perror("connect");
      }
      fprintf(stderr, "%s: Network failure.\n", host);
      close(sock);
      return (-1);
   }
   return (sock);
}

static void
readmeta(sock)
    int             sock;
{
   FILE           *fsock;
   register int    i;
   int		   def_found = 0;

   char           *numstr;
   char            line[BUFSIZ];
   char            misc[BUFSIZ];

   /* let internal buffer code handle the line splitting */
   fsock = fdopen(sock, "r");

   while (fgets(line, BUFSIZ - 1, fsock)) {
      if (sscanf(line, "-h %s -p %d %d %[^\n]",	/* parse line */
		 serverlist[num_servers].address,
		 &serverlist[num_servers].port,
		 &serverlist[num_servers].time,
		 misc) == 4) {

	 serverlist[num_servers].status = -1;

	 /*
	  * I don't have it checking the servers with nobody playing because
	  * the menu window would then be too large to fit on a 1024 x 768
	  * window. :(  - NBT
	  */

	 for (i = 0; i < KEYS; i++) {
	    if ((numstr = strstr(misc, keystrings[i]))) {
	       serverlist[num_servers].status = i;
	       serverlist[num_servers].players = getnumber(numstr, 0);
	       break;
	    }
	 }

	 serverlist[num_servers].flag = misc[strlen(misc) - 1];

	 if (serverlist[num_servers].flag == PARADISE)
	    /* can't handle paradise servers */
	    serverlist[num_servers].status = -1;

	 if (strrchr(misc + 15, 'R'))
	    serverlist[num_servers].RSA_client = 1;
	 else
	    serverlist[num_servers].RSA_client = 0;

	 /* check default */
	 if (strcmpi(serverlist[num_servers].address, serverName) == 0
	     && serverlist[num_servers].port == xtrekPort){
	    serverlist[num_servers].def = 1;
	    def_found = 1;
	 }

	 switch (serverlist[num_servers].status) {
	 case KEY_OPEN:
	 case KEY_WAITQ:
	    num_servers++;	/* It's valid */
	    break;
	 default:
	    /* ignore unless it was our default server */
	    if (serverlist[num_servers].def)
	       num_servers++;
	    break;
	 }
	 if(num_servers == MAX_SERVERS-1){
	    fprintf(stderr, "Too many servers (> %d)\n", MAX_SERVERS);
	    fprintf(stderr, "Client needs fixed.n");
	    break;
	 }
      }
   }

   if(!def_found){
      strcpy(serverlist[num_servers].address, serverName);
      serverlist[num_servers].port = xtrekPort;
      serverlist[num_servers].status = KEY_UNKNOWN;
      serverlist[num_servers].def = 1;
      num_servers ++;
   }
      
   fclose(fsock);
}

static void
parsemeta()
{
   int             sock;

   serverlist = (struct servers *) malloc(sizeof(struct servers) * MAX_SERVERS);
   MZERO((void *) serverlist, sizeof(struct servers) * MAX_SERVERS);

   if ((sock = open_port(metaserver, metaport, 1)) > 0) {
      readmeta(sock);
   } else {
      fprintf(stderr, "Cannot connect to MetaServer (%s , %d)\n",
	      metaserver, metaport);
      exit(0);
   }
}

/* Show the meta server menu window */

static void
metawindow()
{
   register int    i;

   metaWin = W_MakeMenu("MetaServer List", 0, 0, 64, num_servers + 1,
			NULL, 2);
   W_DefineArrowCursor(metaWin);

   for (i = 0; i < num_servers; i++)
      metarefresh(i);

   /* Add quit option */
   W_WriteText(metaWin, 0, num_servers, W_Yellow, "  Quit", 4, 0);

   /* Map window */
   W_MapWindow(metaWin);
}

/* Refresh item i */
static void
metarefresh(i)
    int             i;
{
   char            buf[BUFSIZ];
   int             color = textColor;

   switch (serverlist[i].status) {

   case KEY_OPEN:		/* open */
   case KEY_WAITQ:		/* wait q */
      sprintf(buf, "%s %-40s %12s %-5d %c",
	      serverlist[i].def ? "*" : " ",
	      serverlist[i].address,
	      keystrings[serverlist[i].status],
	      serverlist[i].players,
	      serverlist[i].flag);
      break;
   case KEY_NOBODY:
      sprintf(buf, "%s %-40s EMPTY",
	      serverlist[i].def ? "*" : " ",
	      serverlist[i].address);
      break;

   case KEY_NORESP:
      sprintf(buf, "%s %-40s CAN'T CONNECT",
	      serverlist[i].def ? "*" : " ",
	      serverlist[i].address);
      break;
   
   case KEY_UNKNOWN:
      sprintf(buf, "%s %-40s ???",
	      serverlist[i].def ? "*" : " ",
	      serverlist[i].address);
      break;
   default:
      return;
   }
   if (serverlist[i].def)
      color = W_Green;

   W_WriteText(metaWin, 0, i, color, buf, strlen(buf), 0);
}

/* Check selection to see if was valid. If it was then we have a winner! */

static void
metaaction(data)
    W_Event        *data;
{
   int             sock;

   if ((data->y >= 0) && (data->y < num_servers)) {
      xtrekPort = serverlist[data->y].port;
      serverName = strdup(serverlist[data->y].address);

#ifdef RSA
      RSA_Client = serverlist[data->y].RSA_client;
#endif
      serverAlias = getalias(serverName);

      if ((sock = open_port(serverName, xtrekPort, 1)) <= 0) {
	 serverlist[data->y].status = KEY_NORESP;
	 metarefresh(data->y);
	 close(sock);
      } else {
	 close(sock);
	 metadone();
      }
   } else if (data->y == num_servers)	/* quit button */
      exit(0);
}


/* Unmap the metaWindow */

static void
metadone()
{
   /* Unmap window */
   W_DestroyWindow(metaWin);
   W_Flush();
   if (serverlist)
      free((void *) serverlist);
   quit_meta = 1;
}


/*
 * My own little input() function. I needed this so I don't have to use all
 * the bull in the main input(). Plus to use it I'd have to call mapAll()
 * first and the client would read in the default server and then call it up
 * before I can select a server.
 */

static int
metainput()
{
   W_Event         data;

   while (!quit_meta) {
      W_NextEvent(&data);
      switch ((int) data.type) {
      case W_EV_KEY:
	 if (data.Window == metaWin)
	    metaaction(&data);
	 break;
      case W_EV_BUTTON:
	 if (data.Window == metaWin)
	    metaaction(&data);
	 break;
      case W_EV_EXPOSE:
	 break;
      default:
	 break;
      }
   }
}

void
do_metaserver()
{
   char           *s;

   /* get metaserver defaults */
   if ((s = getdefault("metaserver")))
      metaserver = s;
   metaport = intDefault("metaport", metaport);
#ifdef SHOW_DEFAULTS
   show_defaults("C-startup", "metaserver", metaserver,
		 "Metaserver host to use for -m startup option.");
   show_defaults("C-startup", "metaport", metaport,
		 "Metaserver port to use for -m startup option.");
#endif

   /* get input */
   parsemeta();
   /* put up window */
   metawindow();
   /* wait for input */
   metainput();
}
